@@ -1,5 +1,10 @@
 // Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
 // SPDX-License-Identifier: Apache-2.0
 
+mod select;
+
 #[cfg(feature = "tokio")]
 pub mod tokio;
+
+#[cfg(any(test, feature = "io-testing"))]
+pub mod testing;
