@@ -1,17 +1,11 @@
 // Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
 // SPDX-License-Identifier: Apache-2.0
 
+use super::select::{self, Select};
 use crate::{buffer::default as buffer, features::gso, socket::default as socket};
 use cfg_if::cfg_if;
-use core::{
-    future::Future,
-    pin::Pin,
-    task::{Context, Poll},
-};
-use futures::future::{Fuse, FutureExt};
-use pin_project::pin_project;
 use s2n_quic_core::{
-    endpoint::{CloseError, Endpoint},
+    endpoint::Endpoint,
     event::{self, EndpointPublisher as _},
     inet::{self, SocketAddress},
     path::MaxMtu,
@@ -517,7 +511,7 @@ impl<E: Endpoint<PathHandle = PathHandle>> Instance<E> {
             // pin the wakeups future so we don't have to move it into the Select future.
             tokio::pin!(wakeups);
 
-            let SelectOutcome {
+            let select::Outcome {
                 rx_result,
                 tx_result,
                 timeout_expired,
@@ -568,114 +562,6 @@ impl<E: Endpoint<PathHandle = PathHandle>> Instance<E> {
     }
 }
 
-/// The main event loop future for selecting readiness of sub-tasks
-///
-/// This future ensures all sub-tasks are polled fairly by yielding once
-/// after completing any of the sub-tasks. This is especially important when the TX queue is
-/// flushed quickly and we never get notified of the RX socket having packets to read.
-#[pin_project]
-struct Select<Rx, Tx, Wakeup, Sleep>
-where
-    Rx: Future,
-    Tx: Future,
-    Wakeup: Future,
-    Sleep: Future,
-{
-    #[pin]
-    rx: Fuse<Rx>,
-    rx_out: Option<Rx::Output>,
-    #[pin]
-    tx: Fuse<Tx>,
-    tx_out: Option<Tx::Output>,
-    #[pin]
-    wakeup: Fuse<Wakeup>,
-    #[pin]
-    sleep: Sleep,
-}
-
-impl<Rx, Tx, Wakeup, Sleep> Select<Rx, Tx, Wakeup, Sleep>
-where
-    Rx: Future,
-    Tx: Future,
-    Wakeup: Future,
-    Sleep: Future,
-{
-    #[inline(always)]
-    fn new(rx: Rx, tx: Tx, wakeup: Wakeup, sleep: Sleep) -> Self {
-        Self {
-            rx: rx.fuse(),
-            rx_out: None,
-            tx: tx.fuse(),
-            tx_out: None,
-            wakeup: wakeup.fuse(),
-            sleep,
-        }
-    }
-}
-
-struct SelectOutcome<Rx, Tx> {
-    rx_result: Option<Rx>,
-    tx_result: Option<Tx>,
-    timeout_expired: bool,
-    application_wakeup: bool,
-}
-
-type SelectResult<Rx, Tx> = Result<SelectOutcome<Rx, Tx>, CloseError>;
-
-impl<Rx, Tx, Wakeup, Sleep> Future for Select<Rx, Tx, Wakeup, Sleep>
-where
-    Rx: Future,
-    Tx: Future,
-    Wakeup: Future<Output = Result<usize, CloseError>>,
-    Sleep: Future,
-{
-    type Output = SelectResult<Rx::Output, Tx::Output>;
-
-    fn poll(self: Pin<&mut Self>, cx: &mut Context<'_>) -> Poll<Self::Output> {
-        let this = self.project();
-
-        let mut should_wake = false;
-        let mut application_wakeup = false;
-
-        if let Poll::Ready(wakeup) = this.wakeup.poll(cx) {
-            should_wake = true;
-            application_wakeup = true;
-            if let Err(err) = wakeup {
-                return Poll::Ready(Err(err));
-            }
-        }
-
-        if let Poll::Ready(v) = this.rx.poll(cx) {
-            should_wake = true;
-            *this.rx_out = Some(v);
-        }
-
-        if let Poll::Ready(v) = this.tx.poll(cx) {
-            should_wake = true;
-            *this.tx_out = Some(v);
-        }
-
-        let mut timeout_expired = false;
-
-        if this.sleep.poll(cx).is_ready() {
-            timeout_expired = true;
-            should_wake = true;
-        }
-
-        // if none of the subtasks are ready, return
-        if !should_wake {
-            return Poll::Pending;
-        }
-
-        Poll::Ready(Ok(SelectOutcome {
-            rx_result: this.rx_out.take(),
-            tx_result: this.tx_out.take(),
-            timeout_expired,
-            application_wakeup,
-        }))
-    }
-}
-
 /// A shim for the AsyncFd API
 ///
 /// Tokio only provides the AsyncFd interface for unix platforms so for
@@ -724,7 +610,10 @@ mod async_fd_shim {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use core::convert::TryInto;
+    use core::{
+        convert::TryInto,
+        task::{Context, Poll},
+    };
     use s2n_quic_core::{
         endpoint::{self, CloseError},
         event,
